@@ -19,21 +19,35 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 from __future__ import annotations
-
+import sys
+import os
+if os.name == 'nt':
+    sys.path.insert(0,f'{os.path.abspath(os.path.dirname(os.path.dirname(__file__)))}/wechaty_puppet_itchat')
+    
+sys.path.insert(0,f'{os.path.abspath(os.path.dirname(os.path.dirname(__file__)))}/wechaty_puppet_itchat')
+    
+from itchat  import load_async_itchat
+from itchat.content import (
+    TEXT,
+    MAP,
+    CARD,
+    NOTE,
+    SHARING,
+    PICTURE,
+    RECORDING,
+    VOICE,
+    ATTACHMENT,
+    VIDEO,
+    FRIENDS
+)
 import asyncio
+from distutils import core
 import types
 from typing import Optional, List, Dict
-
-from grpclib.client import Channel
-
 from pyee import AsyncIOEventEmitter  # type: ignore
-
 from wechaty_puppet.schemas.types import PayloadType  # type: ignore
-
+from wechaty_puppet.schemas.event import EventScanPayload,ScanStatus
 from wechaty_puppet import (  # type: ignore
-    EventScanPayload,
-    ScanStatus,
-
     # EventReadyPayload,
     #
     # EventDongPayload,
@@ -59,7 +73,7 @@ from wechaty_puppet import (  # type: ignore
     PuppetOptions,
     MiniProgramPayload,
     UrlLinkPayload,
-
+    watch_dog,
     get_logger
 )
 
@@ -71,26 +85,10 @@ from wechaty_puppet.exceptions import (  # type: ignore
     # WechatyPuppetPayloadError
 )
 
-from src import itchat
-from src.itchat.content import (  # type: ignore
-    TEXT,
-    MAP,
-    CARD,
-    NOTE,
-    SHARING,
-    PICTURE,
-    RECORDING,
-    VOICE,
-    ATTACHMENT,
-    VIDEO,
-    FRIENDS,
-    SYSTEM
-)
-
 # pylint: disable=E0401
 
 log = get_logger('ItChatPuppet')
-
+log.disabled=True
 
 def _map_message_type(message_payload: MessagePayload) -> MessagePayload:
     """
@@ -160,6 +158,7 @@ def _map_message_type(message_payload: MessagePayload) -> MessagePayload:
     return message_payload
 
 
+
 # pylint: disable=R0904
 class PuppetItChat(Puppet):
     """
@@ -177,13 +176,12 @@ class PuppetItChat(Puppet):
             WechatyPuppetConfigurationError: raise Error when configuraiton occur error
         """
         super().__init__(options, name)
-
-        self.channel: Optional[Channel] = None
         self._event_stream: AsyncIOEventEmitter = AsyncIOEventEmitter()
         self.login_user_id: Optional[str] = None
         self.puppet_options = None
         self.puppet = self
-        self.itchat = itchat
+        self.itchat = load_async_itchat()
+        
 
         self.message_container: Dict[str, dict] = {}
 
@@ -196,7 +194,7 @@ class PuppetItChat(Puppet):
         # if response is None:
         #     raise WechatyPuppetGrpcError('can"t get room_list response')
         # return response.ids
-        return [i['UserName'] for i in self.itchat.get_chatrooms()]
+        return [i['UserName'] for i in  self.itchat.get_chatrooms()]
 
     async def message_image(self, message_id: str, image_type: ImageType = 3
                             ) -> FileBox:
@@ -217,6 +215,13 @@ class PuppetItChat(Puppet):
         # file_stream = reduce(lambda pre, cu: pre + cu, file_chunk_data)
         # file_box = FileBox.from_stream(file_stream, name=name)
         # return file_box
+        if message_id not in self.message_container:
+            raise ValueError('img message not found')
+        msg = self.message_container[message_id]
+        get_pic = await msg['Text']
+        await get_pic(msg['fileName'])
+        file_box = FileBox.from_file(path=msg['fileName'], name=msg['fileName'])
+        return file_box
 
     def on(self, event_name: str, caller):
         """
@@ -242,7 +247,7 @@ class PuppetItChat(Puppet):
         get contact list
         :return:
         """
-        return [i['UserName'] for i in self.itchat.get_friends()]
+        return [i['UserName'] for i in self.itchat.get_friends(update=True)]
 
     async def tag_contact_delete(self, tag_id: str) -> None:
         """
@@ -317,7 +322,16 @@ class PuppetItChat(Puppet):
         # response = await self.puppet_stub.message_send_text(
         #     conversation_id=conversation_id,
         #     text=message, mentonal_ids=mention_ids)
-        response = await self.itchat.send_msg(message, toUserName=conversation_id)
+        response = await self.itchat.send_msg(msg = message, toUserName=conversation_id)
+        msg={}
+        msg['MsgId'] = response['MsgID']
+        msg['FromUserName'] = ''
+        msg['ToUserName'] = ''
+        msg['Type'] = 'Text'
+        msg['Text'] = message
+        msg['FileName'] = ''
+        msg['CreateTime']= ''
+        self.message_container[response['MsgID']] = msg
         return response['MsgID']
 
     async def message_send_contact(self, contact_id: str,
@@ -344,18 +358,41 @@ class PuppetItChat(Puppet):
         :param file:
         :return:
         """
+        file_path = file.name
         if file.name.endswith('.jpg') or \
                 file.name.endswith('.jpeg') or \
                 file.name.endswith('.png') or \
                 file.name.endswith('.gif') or \
                 file.name.endswith('.bmp'):
-            file_path = file.name
             await file.to_file(file_path=file_path, overwrite=True)
             response = await self.itchat.send_image(fileDir=file_path, toUserName=conversation_id)
+        elif file.name.endswith(('.mp4','.MP4'))  :
+            await file.to_file(file_path=file_path, overwrite=True)
+            response = await self.itchat.send_video(fileDir=file_path, toUserName=conversation_id)
+            msg={}
+            msg['MsgId'] = response['MsgID']
+            msg['FromUserName'] = ''
+            msg['ToUserName'] = ''
+            msg['Type'] = 'Video'
+            msg['Text'] = ''
+            msg['FileName'] = file_path
+            msg['CreateTime']= ''
+            self.message_container[response['MsgID']] = msg
             return response['MsgID']
-        file_path = file.name
-        await file.to_file(overwrite=True)
-        response = await self.itchat.send_file(fileDir=file_path, toUserName=conversation_id)
+        else:
+            file_path = file.name
+            await file.to_file(overwrite=True)
+            response = await self.itchat.send_file(fileDir=file_path, toUserName=conversation_id)
+        msg={}
+        print(response)
+        msg['MsgId'] = response['MsgID']
+        msg['FromUserName'] = ''
+        msg['ToUserName'] = ''
+        msg['Type'] = 'Attachment'
+        msg['Text'] = ''
+        msg['FileName'] = file_path
+        msg['CreateTime']= ''
+        self.message_container[response['MsgID']] = msg
         return response['MsgID']
 
     async def message_send_url(self, conversation_id: str, url: str) -> str:
@@ -419,7 +456,6 @@ class PuppetItChat(Puppet):
             raise ValueError('message not found')
         msg = self.message_container[message_id]
         msg_type = msg['Type']
-
         if msg_type == 'Text':
             type_ = MessageType.MESSAGE_TYPE_TEXT
         elif msg_type == 'Map':
@@ -437,14 +473,23 @@ class PuppetItChat(Puppet):
         else:
             type_ = MessageType.MESSAGE_TYPE_UNSPECIFIED
 
-        message_payload = MessagePayload(id=msg['MsgId'],
-                                         type=type_,
-                                         from_id=msg['FromUserName'],
-                                         filename=msg['FileName'],
-                                         text=msg['Text'],
-                                         timestamp=msg['CreateTime'],
-                                         to_id=msg['ToUserName'])
-
+        if '@@' in msg['FromUserName']:
+            message_payload = MessagePayload(id=msg['MsgId'],
+                                             type=type_,
+                                             from_id=msg['ToUserName'],
+                                             room_id=msg['FromUserName'],
+                                             filename=msg['FileName'],
+                                             text=msg['Text'],
+                                             timestamp=msg['CreateTime'],
+                                             to_id=msg['ToUserName'])
+        else:
+            message_payload = MessagePayload(id=msg['MsgId'],
+                                             type=type_,
+                                             from_id=msg['FromUserName'],
+                                             filename=msg['FileName'],
+                                             text=msg['Text'],
+                                             timestamp=msg['CreateTime'],
+                                             to_id=msg['ToUserName'])
         return message_payload
 
     async def message_forward(self, to_id: str, message_id: str):
@@ -493,7 +538,7 @@ class PuppetItChat(Puppet):
         #         name = stream.file_box_chunk.name
 
         if message_id not in self.message_container:
-            raise ValueError('message not found')
+            raise ValueError('file message not found')
         msg = self.message_container[message_id]
         get_pic = await msg['Text']
         await get_pic(msg['fileName'])
@@ -507,7 +552,7 @@ class PuppetItChat(Puppet):
         :return:
         """
         if message not in self.message_container:
-            raise ValueError('message not found')
+            raise ValueError('emoticon message not found')
         msg = self.message_container[message]
         get_pic = await msg['Text']
         await get_pic(msg['fileName'])
@@ -520,10 +565,11 @@ class PuppetItChat(Puppet):
         :param message_id:
         :return:
         """
-        # response = await self.puppet_stub.message_contact(id=message_id)
+        
+        # response = await self.puppet.message_contact(id=message_id)
         # return response.id
         if message_id not in self.message_container:
-            raise ValueError('message not found')
+            raise ValueError('contact message not found')
         msg = self.message_container[message_id]
         return msg['RecommendInfo']['UserName']
 
@@ -534,7 +580,7 @@ class PuppetItChat(Puppet):
         :return:
         """
         if message_id not in self.message_container:
-            raise ValueError('message not found')
+            raise ValueError('url message not found')
         msg = self.message_container[message_id]
         return UrlLinkPayload(
             url=msg['Url'],
@@ -598,9 +644,9 @@ class PuppetItChat(Puppet):
         """
         # response = await self.puppet_stub.contact_payload(id=contact_id)
         # return response
-        contact_list = self.itchat.get_contact(update=True)
+        contact_list = self.itchat.get_friends(update=True)
         for c in contact_list:
-            if c['UserName'] == contact_id:
+            if c.get('UserName') == contact_id:
                 return ContactPayload(
                     id=c['UserName'],
                     gender=c['Sex'],
@@ -675,8 +721,7 @@ class PuppetItChat(Puppet):
         #     contact_id=contact_id,
         #     hello=hello
         # )
-        await self.itchat.Core.add_friend(self.itchat.Core, userName=contact_id,
-                                          status=2, verifyContent=hello)
+        await self.itchat.accept_friend( userName=contact_id,v4='')
 
     async def friendship_payload(self, friendship_id: str,
                                  payload: Optional[FriendshipPayload] = None
@@ -700,7 +745,7 @@ class PuppetItChat(Puppet):
         :param friendship_id:
         :return:
         """
-        await self.itchat.Core.add_friend(self.itchat.Core, userName=friendship_id, status=3)
+        await self.itchat.accept_friend( userName=friendship_id,v4='')
 
     async def room_create(self, contact_ids: Optional[List[str]], topic: Optional[str] = None
                           ) -> str:
@@ -717,7 +762,8 @@ class PuppetItChat(Puppet):
         # return response.id
         return self.itchat.create_chatroom(memberList=contact_ids, topic=topic)['ChatRoomName']
 
-    async def room_search(self, query: RoomQueryFilter = None) -> List[str]:
+    # async def room_search(self, query: RoomQueryFilter = None) -> List[str]:
+    async def room_search(self) -> List[str]:
         """
         find the room_ids
         search room
@@ -726,18 +772,8 @@ class PuppetItChat(Puppet):
         """
         # room_list_response = await self.puppet_stub.room_list()
         # return room_list_response.ids
-        room_list = self.itchat.get_chatrooms
-        ret_room_list = []
-        if query:
-            for r in room_list:
-                if query.id:
-                    if r['UserName'] == query.id:
-                        ret_room_list.append(r['UserName'])
-                        continue
-                elif query.topic:
-                    if r['RemarkName'] == query.topic:
-                        ret_room_list.append(r['UserName'])
-        return ret_room_list
+        return [i['UserName'] for i in self.itchat.get_chatrooms(update=True)]
+
 
     async def room_invitation_payload(self,
                                       room_invitation_id: str,
@@ -822,6 +858,18 @@ class PuppetItChat(Puppet):
         """
         # response = await self.puppet_stub.room_payload(id=room_id)
         # return response
+        room_list = self.itchat.get_chatrooms(update=True)
+        for r in room_list:
+            print(r)
+            if r['UserName'] == room_id:
+                member_ids = [c['UserName'] for c in r['MemberList']]
+                return RoomPayload(
+                    id=r['UserName'],
+                    topic=r['NickName'],
+                    avatar=r['HeadImgUrl'],
+                    owner_id=r.get('ChatRoomOwner'),
+                    admin_ids= [r.get('ChatRoomOwner')],
+                    member_ids=member_ids)
 
     async def room_members(self, room_id: str) -> List[str]:
         """
@@ -965,6 +1013,7 @@ class PuppetItChat(Puppet):
         """
         log.info('stop()')
         self._event_stream.remove_all_listeners()
+        return None
 
     async def logout(self):
         """
@@ -1000,8 +1049,10 @@ class PuppetItChat(Puppet):
         :return:
         """
         # log.debug('send ding info to itchat server ...')
-        #
-        # await self.puppet_stub.ding(data=data)
+        #await self.puppet.ding(data='dong')
+        food =watch_dog.WatchdogFood(timeout=3)
+        if data:
+            data.feed(food)
 
     # pylint: disable=R0912,R0915
     async def _listen_for_event(self):
@@ -1031,11 +1082,14 @@ class PuppetItChat(Puppet):
             await asyncio.sleep(0.1)
 
         await self.itchat.auto_login(
-            enableCmdQR=True,
-            qrCallback=on_scan,
+            hotReload=True,
+            statusStorageDir='itchat.pkl',
             EventScanPayload=EventScanPayload,
             ScanStatus=ScanStatus,
             event_stream=self._event_stream,
+            enableCmdQR=True,
+            picDir=None,
+            qrCallback=on_scan,
             loginCallback=on_logined,
             exitCallback=on_logout
         )
@@ -1053,7 +1107,6 @@ class PuppetItChat(Puppet):
                 timestamp=msg['CreateTime'],
                 to_id=msg['ToUserName']
             )
-
             self._event_stream.emit('message', event_message_payload)
             await asyncio.sleep(0.1)
 
@@ -1108,31 +1161,53 @@ class PuppetItChat(Puppet):
             self._event_stream.emit('message', event_message_payload)
             await asyncio.sleep(0.1)
 
-        @itchat.msg_register(FRIENDS)
+        @self.itchat.msg_register(FRIENDS)
         def add_friend(msg):
+            self.message_container[msg['MsgId']] = msg
+            log.info(f'receive file message info <{msg.user}>')
             msg.user.verify()
-            msg.user.send('Nice to meet you!')
 
-        @itchat.msg_register(SYSTEM)
-        def handle_system_msg(msg):
-            log.info(f'receive system message info <{msg["Text"]}>')
+        # @itchat.msg_register(SYSTEM)
+        # def handle_system_msg(msg):
+        #     log.info(f'receive system message info <{msg["Text"]}>')
 
         message_container = self.message_container.copy()
-
         async def run(self, event_stream, payload):
             async def reply_fn():
                 try:
-                    while self.alive:
+                    while self.alive :
+                        await asyncio.sleep(2)
                         await self.configured_reply(event_stream=event_stream, payload=payload,
-                                                    message_container=message_container)
+                                                        message_container=message_container)
+                        
                 except KeyboardInterrupt:
                     if self.useHotReload:
                         await self.dump_login_status()
                     self.alive = False
+            await reply_fn()
 
-            while True:
-                await asyncio.sleep(0.5)
-                await reply_fn()
+            #v3
+            # 定义一个专门创建事件循环loop的函数，在另一个线程中启动它
+            # def start_loop(loop):
+            #     asyncio.set_event_loop(loop)
+            #     loop.run_forever()
+            # new_loop = asyncio.new_event_loop()  # 在当前线程下创建时间循环，（未启用），在start_loop里面启动它
+            # import threading
+            # t = threading.Thread(target=start_loop, args=(new_loop,))  # 通过当前线程开启新的线程去启动事件循环
+            # t.start()
 
-        self.itchat.run = types.MethodType(run, self.itchat.originInstance)
+            # asyncio.run_coroutine_threadsafe(reply_fn(), new_loop) 
+            # def new_thread():
+            #     async def main():           
+            #         await reply_fn()
+            #     asyncio.run(reply_fn())
+            # import threading
+            # replyThread = threading.Thread(target=new_thread)
+            # replyThread.setDaemon(True)
+            # replyThread.start() 
+            # while True:
+            #     await asyncio.sleep(1)
+            #     await reply_fn()
+            
+        self.itchat.run = types.MethodType(run, self.itchat)#self.itchat.originInstance)
         await self.itchat.run(event_stream=self._event_stream, payload=EventMessagePayload)
